<?php

namespace Drupal\Tests\force_password_change\Controller;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Tests\UnitTestCase;
use Drupal\force_password_change\Controller\ForcePasswordChangeController;

/**
 * @coversDefaultClass \Drupal\force_password_change\Controller\ForcePasswordChangeController
 * @group force_password_change
 */
class ForcePasswordChangeControllerTest extends UnitTestCase
{
	protected $userStub;
	protected $role;
	protected $roleStorage;
	protected $entityManager;
	protected $formBuilder;
	protected $dateFormatter;
	protected $currentUser;
	protected $userData;
	protected $forcePasswordChangeService;

	public function setUp()
	{
		$this->userStub = $this->getMockBuilder('\Drupal\user\UserInterface')
			->disableOriginalConstructor()
			->getMock();
		$this->userStub->expects($this->any())
			->method('id')
			->willReturn(1);
		$this->userStub->expects($this->any())
			->method('getDisplayName')
			->willReturn('user1');
		$this->userStub->expects($this->any())
			->method('getCreatedTime')
			->willReturn(1112223334);

		$this->role = $this->getMockBuilder('\Drupal\user\Entity\Role')
			->disableOriginalConstructor()
			->getMock();
		$this->role->expects($this->any())
			->method('id')
			->willReturn('authenticated');

		$this->roleStorage = $this->getMockBuilder('\Drupal\user\RoleStorage')
			->disableOriginalConstructor()
			->getMock();
		$this->roleStorage->expects($this->any())
			->method('load')
			->will($this->returnValueMap([
				['authenticated', $this->role],
			]));

		$this->entityManager = $this->getMockBuilder('Drupal\Core\Entity\EntityManager')
			->disableOriginalConstructor()
			->getMock();
		$this->entityManager->expects($this->any())
			->method('getStorage')
			->with(NULL)
			->willReturn($this->roleStorage);

		$container = new ContainerBuilder();
		$container->set('entity.manager', $this->entityManager);
		$container->set('string_translation', $this->getStringTranslationStub());
		\Drupal::setContainer($container);

		$this->formBuilder = $this->getMockBuilder('Drupal\Core\Form\FormBuilderInterface')
			->disableOriginalConstructor()
			->getMock();
		$this->formBuilder->expects($this->any())
			->method('getForm')
			->willReturn([]);

		$this->dateFormatter = $this->getMockBuilder('Drupal\Core\Datetime\DateFormatter')
			->disableOriginalConstructor()
			->getMock();

		$this->dateFormatter->expects($this->any())
			->method('format')
			->willReturn('dummy date');

		$this->currentUser = $this->getMockBuilder('Drupal\Core\Session\AccountProxy')
			->disableOriginalConstructor()
			->getMock();
	}

	/**
	 * @covers ::adminPage
	 */
	public function testAdminPage()
	{
		$configFactory = $this->getConfigFactory();
		$userData = $this->getUserData();
		$forcePasswordChangeService = $this->getForcePasswordChangeService();

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$admin_page = $controller->adminPage();

		$this->assertTrue(is_array($admin_page), 'Admin page is an array');
		$this->assertArrayHasKey('#prefix', $admin_page, 'Admin page has key #prefix');
		$this->assertEquals($admin_page['#prefix'], '<div id="force_password_change_admin_page">', 'Admin page #prefix is correct');
		$this->assertArrayHasKey('#suffix', $admin_page, 'Admin page has key #suffix');
		$this->assertEquals($admin_page['#suffix'], '</div>', 'Admin page #suffix is correct');
		$this->assertArrayHasKey('form', $admin_page, 'Admin page has key form');
	}

	/**
	 * @covers ::roleListPage
	 */
	public function testRoleListPage()
	{
		$configFactory = $this->getConfigFactory();
		$userData = $this->getUserData();
		$forcePasswordChangeService = $this->getForcePasswordChangeService();

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('anonymous');

		$this->assertTrue(is_array($role_list_page), '$role_list_page is an array for anonymous role');
		$this->assertArrayHasKey('#prefix', $role_list_page, '$role_list_page has key #prefix for anonymous role');
		$this->assertEquals($role_list_page['#prefix'], '<div id="force_password_change_role_page">', '$role_list_page #prefix is correct');
		$this->assertArrayHasKey('#suffix', $role_list_page, '$role_list_page has key #suffix');
		$this->assertEquals($role_list_page['#suffix'], '</div>', '$role_list_page #suffix is correct');
		$this->assertArrayHasKey('invalid_role', $role_list_page, '$role_list_page has invalid_role_key for anonymous role');
		$this->assertArrayNotHasKey('form', $role_list_page, '$role_list_page does not have key form for anonymous role');

		$configFactory = $this->getConfigFactory([
			[
				'with1' => 'force_password_change.installation_date',
				'with2' => 'unix_date',
				'returns' => 1234567890,
			],
			[
				'with1' => 'force_password_change.first_time_uids',
				'with2' => 'uids',
				'returns' => FALSE,
			]
		]);

		$userData = $this->getUserData();

		$forcePasswordChangeService = $this->getForcePasswordChangeService([
			[
				'method' => 'getPendingUsersForRole',
				'returns' => [],
			],
			[
				'method' => 'getNonPendingUsersForRole',
				'returns' => [],
			],
		]);

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('authenticated');

		$this->assertTrue(is_array($role_list_page), '$role_list_page is an array for authenticated user');
		$this->assertArrayHasKey('pending_users_table', $role_list_page, '$role_list_page has key pending_users_table for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertArrayHasKey('table', $role_list_page['pending_users_table'], '$role_list_page[pending_users_table] has key table for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertArrayHasKey('#type', $role_list_page['pending_users_table']['table'], '$role_list_page[pending_users_table][table] has key #type for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertEquals('table', $role_list_page['pending_users_table']['table']['#type'], 'Pending roles table element type is table');
		$this->assertCount(0, $role_list_page['pending_users_table']['table']['#rows'], 'Pending roles table element rows is empty array');
		$this->assertArrayHasKey('nonpending_users_table', $role_list_page, '$role_list_page has pending_users_table for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertArrayHasKey('table', $role_list_page['nonpending_users_table'], '$role_list_page[nonpending_users_table] has key table for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertArrayHasKey('#type', $role_list_page['nonpending_users_table']['table'], '$role_list_page[nonpending_users_table][table] has key #type for authenticated role ' . print_r(array_keys($role_list_page), TRUE));
		$this->assertEquals('table', $role_list_page['nonpending_users_table']['table']['#type'], 'nonpending_users_table table element type is table');
		$this->assertCount(0, $role_list_page['nonpending_users_table']['table']['#rows'], 'nonpending_users_table table element rows is empty array');
		$this->assertArrayHasKey('form', $role_list_page, '$role_list_page has key form for authenticated role ' . print_r(array_keys($role_list_page), TRUE));

		$configFactory = $this->getConfigFactory([
			[
				'with1' => 'force_password_change.installation_date',
				'with2' => 'unix_date',
				'returns' => 1234567890,
			],
			[
				'with1' => 'force_password_change.first_time_uids',
				'with2' => 'uids',
				'returns' => FALSE,
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => FALSE,
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => FALSE,
			],
		]);

		$userData = $this->getUserData([
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => FALSE,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => 11223344,
			],
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => FALSE,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => 11223344,
			],
		]);

		$forcePasswordChangeService = $this->getForcePasswordChangeService([
			[
				'method' => 'getPendingUsersForRole',
				'returns' => [$this->userStub],
			],
			[
				'method' => 'getNonPendingUsersForRole',
				'returns' => [$this->userStub],
			],
		]);

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('authenticated');

		$this->assertArrayHasKey('#prefix', $role_list_page, '$role_list_page has #prefix key');
		$this->assertEquals($role_list_page['#prefix'], '<div id="force_password_change_role_page">', '$role_list_page[#prefix] is correct');
		$this->assertArrayHasKey('#suffix', $role_list_page, '$role_list_page has #suffix key');
		$this->assertEquals($role_list_page['#suffix'], '</div>', '$role_list_page[#suffix] is correct');

		$this->assertArrayHasKey('pending_users_table', $role_list_page, '$role_list_page has pending_users_table key when pending users exist');
		$this->assertArrayHasKey('header', $role_list_page['pending_users_table'], '$role_list_page[pending_users_table] has header key when pending users exist');
		$this->assertArrayHasKey('table', $role_list_page['pending_users_table'], '$role_list_page[pending_users_table] has table key when pending users exist');
		$this->assertArrayHasKey('pager', $role_list_page['pending_users_table'], '$role_list_page[pending_users_table] has pager key when pending users exist');
		$this->assertArrayHasKey('#rows', $role_list_page['pending_users_table']['table'], '$role_list_page[pending_users_table][table] has #rows key when pending users exist');
		$this->assertArrayHasKey('0', $role_list_page['pending_users_table']['table']['#rows'], '$role_list_page[pending_users_table][table][#rows] has 0 key when pending users exist');
		$this->assertArrayHasKey('0', $role_list_page['pending_users_table']['table']['#rows'][0], '$role_list_page[pending_users_table][table][#rows][0] has 0 key when pending users exist');
		$this->assertTrue(is_string($role_list_page['pending_users_table']['table']['#rows'][0][0]), '$role_list_page[pending_users_table][table][#rows][0][0] is a string when the viewer does not have permission to access user profiles');
		$this->assertEquals('user1', $role_list_page['pending_users_table']['table']['#rows'][0][0], '$role_list_page[pending_users_table][table][#rows][0][0] is correct username when the viewer does not have permission to access user profiles');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['pending_users_table']['table']['#rows'][0][1], '$role_list_page[pending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup');
		$this->assertEquals('Never', $role_list_page['pending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[pending_users_table][table][#rows][0][1] is equal to "never" when the user has never been forced to change their password');
		$this->assertTrue(is_string($role_list_page['pending_users_table']['table']['#rows'][0][2]), '$role_list_page[pending_users_table][table][#rows][0][2] is a string');
		$this->assertEquals('dummy date', $role_list_page['pending_users_table']['table']['#rows'][0][2], '$role_list_page[pending_users_table][table][#rows][0][2] returns a formatted date');

		$this->assertArrayHasKey('nonpending_users_table', $role_list_page, '$role_list_page has nonpending_users_table key when pending users exist');
		$this->assertArrayHasKey('header', $role_list_page['nonpending_users_table'], '$role_list_page[nonpending_users_table] has header key when pending users exist');
		$this->assertArrayHasKey('table', $role_list_page['nonpending_users_table'], '$role_list_page[nonpending_users_table] has table key when pending users exist');
		$this->assertArrayHasKey('pager', $role_list_page['nonpending_users_table'], '$role_list_page[nonpending_users_table] has pager key when pending users exist');
		$this->assertArrayHasKey('#rows', $role_list_page['nonpending_users_table']['table'], '$role_list_page[nonpending_users_table][table] has #rows key when pending users exist');
		$this->assertArrayHasKey('0', $role_list_page['nonpending_users_table']['table']['#rows'], '$role_list_page[nonpending_users_table][table][#rows] has 0 key when pending users exist');
		$this->assertArrayHasKey('0', $role_list_page['nonpending_users_table']['table']['#rows'][0], '$role_list_page[nonpending_users_table][table][#rows][0] has 0 key when pending users exist');
		$this->assertTrue(is_string($role_list_page['nonpending_users_table']['table']['#rows'][0][0]), '$role_list_page[nonpending_users_table][table][#rows][0][0] is a string when the viewer does not have permission to access user profiles');
		$this->assertEquals('user1', $role_list_page['nonpending_users_table']['table']['#rows'][0][0], '$role_list_page[nonpending_users_table][table][#rows][0][0] is correct username when the viewer does not have permission to access user profiles');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['nonpending_users_table']['table']['#rows'][0][1], '$role_list_page[nonpending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup');
		$this->assertEquals('Never', $role_list_page['nonpending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[nonpending_users_table][table][#rows][0][1] is equal to "never" when the user has never been forced to change their password');
		$this->assertTrue(is_string($role_list_page['nonpending_users_table']['table']['#rows'][0][2]), '$role_list_page[nonpending_users_table][table][#rows][0][2] is a string');
		$this->assertEquals('dummy date', $role_list_page['nonpending_users_table']['table']['#rows'][0][2], '$role_list_page[nonpending_users_table][table][#rows][0][2] returns a formatted date');

		$this->assertArrayHasKey('form', $role_list_page, '$role_list_page has form array key');
		$this->assertTrue(is_array($role_list_page['form']), '$role_list_page[form] is an array');

		$configFactory = $this->getConfigFactory([
			[
				'with1' => 'force_password_change.installation_date',
				'with2' => 'unix_date',
				'returns' => 1234567890,
			],
			[
				'with1' => 'force_password_change.first_time_uids',
				'with2' => 'uids',
				'returns' => FALSE,
			],
		]);

		$userData = $this->getUserData([
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 1234567890,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 1234567890,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
		]);

		$forcePasswordChangeService = $this->getForcePasswordChangeService([
			[
				'method' => 'getPendingUsersForRole',
				'returns' => [$this->userStub],
			],
			[
				'method' => 'getNonPendingUsersForRole',
				'returns' => [$this->userStub],
			],
		]);

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('authenticated');

		$this->assertTrue(is_string($role_list_page['nonpending_users_table']['table']['#rows'][0][1]), '$role_list_page[nonpending_users_table][table][#rows][0][1] is a string when the user has had a force password change');
		$this->assertEquals('dummy date', $role_list_page['nonpending_users_table']['table']['#rows'][0][1], '$role_list_page[nonpending_users_table][table][#rows][0][1] is a date when the user has had a forced password change');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['nonpending_users_table']['table']['#rows'][0][2], '$role_list_page[nonpending_users_table][table][#rows][0][2] is a translated string when the user has changed their password.');
		$this->assertEquals('Never', $role_list_page['nonpending_users_table']['table']['#rows'][0][2]->getUntranslatedString(), '$role_list_page[nonpending_users_table][table][#rows][0][2] returns the correct string when a user has changed their password');

		$this->currentUser->expects($this->any())
			->method('hasPermission')
			->with('access user profiles')
			->willReturn(TRUE);

		$configFactory = $this->getConfigFactory([
			[
				'with1' => 'force_password_change.installation_date',
				'with2' => 'unix_date',
				'returns' => 1112223333,
			],
			[
				'with1' => 'force_password_change.first_time_uids',
				'with2' => 'uids',
				'returns' => FALSE,
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => TRUE,
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => TRUE,
			],
		]);

		$userData = $this->getUserData([
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 0,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 0,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
		]);

		$forcePasswordChangeService = $this->getForcePasswordChangeService([
			[
				'method' => 'getPendingUsersForRole',
				'returns' => [$this->userStub],
			],
			[
				'method' => 'getNonPendingUsersForRole',
				'returns' => [$this->userStub],
			],
		]);

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('authenticated');

		$this->assertInstanceOf('Drupal\Core\Link', $role_list_page['pending_users_table']['table']['#rows'][0][0], '$role_list_page[pending_users_table][table][#rows][0][0] is an instance of Link when viewer has access user profiles permission');
		$this->assertInstanceOf('Drupal\Core\Url', $role_list_page['pending_users_table']['table']['#rows'][0][0]->getUrl(), '$role_list_page[pending_users_table][table][#rows][0][0] URL is an instance of Link when viewer has access user profiles permission');
		$this->assertEquals('entity.user.canonical', $role_list_page['pending_users_table']['table']['#rows'][0][0]->getUrl()->getRouteName(), '$role_list_page[pending_users_table][table][#rows][0][0] route is correct');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['pending_users_table']['table']['#rows'][0][1], '$role_list_page[pending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup');
		$this->assertEquals('First login', $role_list_page['pending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[pending_users_table][table][#rows][0][1] is equal to First Login when admin settings are set to first login');
		$this->assertInstanceOf('Drupal\Core\Link', $role_list_page['nonpending_users_table']['table']['#rows'][0][0], '$role_list_page[nonpending_users_table][table][#rows][0][0] is an instance of Link when viewer has access user profiles permission');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['nonpending_users_table']['table']['#rows'][0][1], '$role_list_page[nonpending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup');
		$this->assertInstanceOf('Drupal\Core\Url', $role_list_page['nonpending_users_table']['table']['#rows'][0][0]->getUrl(), '$role_list_page[nonpending_users_table][table][#rows][0][0] URL is an instance of Link when viewer has access user profiles permission');
		$this->assertEquals('entity.user.canonical', $role_list_page['nonpending_users_table']['table']['#rows'][0][0]->getUrl()->getRouteName(), '$role_list_page[nonpending_users_table][table][#rows][0][0] route is correct');
		$this->assertEquals('First login', $role_list_page['nonpending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[nonpending_users_table][table][#rows][0][1] is equal to First Login when admin settings are set to first login');

		$configFactory = $this->getConfigFactory([
			[
				'with1' => 'force_password_change.installation_date',
				'with2' => 'unix_date',
				'returns' => 1112223333,
			],
			[
				'with1' => 'force_password_change.first_time_uids',
				'with2' => 'uids',
				'returns' => [1],
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => FALSE,
			],
			[
				'with1' => 'force_password_change.first_time_login_password_change',
				'with2' => 'change',
				'returns' => FALSE,
			],
		]);

		$userData = $this->getUserData([
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 0,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
			[
				'name' => 'last_force',
				'uid' => 1,
				'returns' => 0,
			],
			[
				'name' => 'last_change',
				'uid' => 1,
				'returns' => FALSE,
			],
		]);

		$forcePasswordChangeService = $this->getForcePasswordChangeService([
			[
				'method' => 'getPendingUsersForRole',
				'returns' => [$this->userStub],
			],
			[
				'method' => 'getNonPendingUsersForRole',
				'returns' => [$this->userStub],
			],
		]);

		$controller = new ForcePasswordChangeController($this->formBuilder, $this->dateFormatter, $this->currentUser, $configFactory, $userData, $forcePasswordChangeService);
		$role_list_page = $controller->roleListPage('authenticated');

		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['pending_users_table']['table']['#rows'][0][1], '$role_list_page[pending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup when user has UID set');
		$this->assertEquals('First login', $role_list_page['pending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[pending_users_table][table][#rows][0][1] is equal to First Login when user has UID set');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $role_list_page['nonpending_users_table']['table']['#rows'][0][1], '$role_list_page[nonpending_users_table][table][#rows][0][1] is an instance of TranslatableMarkup when user has UID set');
		$this->assertEquals('First login', $role_list_page['nonpending_users_table']['table']['#rows'][0][1]->getUntranslatedString(), '$role_list_page[nonpending_users_table][table][#rows][0][1] is equal to First Login when user has UID set');
	}

	private function getConfigFactory($expectations = [])
	{
		$configFactory = $this->getMockBuilder('Drupal\Core\Config\ConfigFactory')
			->disableOriginalConstructor()
			->getMock();

		if(count($expectations))
		{
			$index = 0;
			foreach($expectations as $expectation)
			{
				$configFactory->expects($this->at($index))
					->method('get')
					->with($expectation['with1'])
					->willReturn($configFactory);
				$index++;
				$configFactory->expects($this->at($index))
					->method('get')
					->with($expectation['with2'])
					->willReturn($expectation['returns']);
				$index++;
			}
		}

		return $configFactory;
	}

	private function getUserData($expectations = [])
	{
		$userData = $this->getMockBuilder('Drupal\user\UserData')
			->disableOriginalConstructor()
			->getMock();

		if(count($expectations))
		{
			$index = 0;
			foreach($expectations as $expectation)
			{
				$userData->expects($this->at($index))
					->method('get')
					->with('force_password_change', $expectation['uid'], $expectation['name'])
					->willReturn($expectation['returns']);

				$index++;
			}
		}

		return $userData;
	}

	private function getForcePasswordChangeService($expectations = [])
	{
		$forcePasswordChangeService = $this->getMockBuilder('Drupal\force_password_change\Service\ForcePasswordChangeService')
			->disableOriginalConstructor()
			->getMock();

		if(count($expectations))
		{
			$index = 0;
			foreach($expectations as $expectation)
			{
				$forcePasswordChangeService->expects($this->at($index))
					->method($expectation['method'])
					->willReturn($expectation['returns']);

				$index++;
			}
		}

		return $forcePasswordChangeService;
	}
}
