<?php

namespace Drupal\Tests\force_password_change\Form;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Form\FormState;
use Drupal\Tests\Core\Form\FormTestBase;
use Drupal\force_password_change\Form\AdminForm;

/**
 * @coversDefaultClass \Drupal\force_password_change\Form\AdminForm
 * @group force_password_change
 */
class AdminFormTest extends FormTestBase
{
	/**
	 * The AdminForm
	 */
	protected $form;

	/**
	 * The Config Factory
	 */
	protected $configFactory;

	/**
	 * The Force Password Change Service
	 */
	protected $forcePasswordChangeService;

	/**
	 * {@inheritdoc}
	 */
	public function setUp()
	{
		parent::setUp();

		// Create a dummy container.
		$container = new ContainerBuilder();

		$container->set('string_translation', $this->getStringTranslationStub());

		$this->configFactory = $this->getMockBuilder('\Drupal\Core\Config\ConfigFactoryInterface')
			->disableOriginalConstructor()
			->getMock();

		$container->set('config.factory', $this->configFactory);

		$this->forcePasswordChangeService = $this->getMockBuilder('\Drupal\force_password_change\Service\ForcePasswordChangeServiceInterface')
			->disableOriginalConstructor()
			->getMock();
		$authenticated_role_info = new \stdClass();
		$authenticated_role_info->rid = 'authenticated';
		$authenticated_role_info->expiry = 60 * 60 * 24;
		$authenticated_role_info->weight = 0;
		$this->forcePasswordChangeService->expects($this->any())
			->method('getRoleExpiryDates')
			->willReturn([$authenticated_role_info]);
		$this->forcePasswordChangeService->expects($this->any())
			->method('getUserCountForRole')
			->willReturn(1);
		$this->forcePasswordChangeService->expects($this->any())
			->method('getPendingUsersForRole')
			->with('authenticated', TRUE)
			->willReturn(1);

		$container->set('force_password_change.service', $this->forcePasswordChangeService);

		$role = $this->getMockBuilder('\Drupal\user\Entity\Role')
			->disableOriginalConstructor()
			->getMock();
		$role->expects($this->any())
			->method('id')
			->willReturn('authenticated');
		$role->expects($this->any())
			->method('label')
			->willReturn('Authenticated');

		$roleStorage = $this->getMockBuilder('\Drupal\user\RoleStorageInterface')
			->disableOriginalConstructor()
			->getMock();
		$roleStorage->expects($this->any())
			->method('load')
			->will($this->returnValueMap([
				['authenticated', $role],
			]));
		$roleStorage->expects($this->any())
			->method('loadMultiple')
			->willReturn(['authenticated' => $role]);

		$entityManager = $this->getMockBuilder('Drupal\Core\Entity\EntityManagerInterface')
			->disableOriginalConstructor()
			->getMock();
		$entityManager->expects($this->any())
			->method('getStorage')
			->with(NULL)
			->willReturn($roleStorage);

		$container->set('entity.manager', $entityManager);

		$this->urlGenerator = $this->getMock('Drupal\Core\Routing\UrlGeneratorInterface');
		$this->urlGenerator->expects($this->any())
			->method('generateFromRoute')
			->willReturn('/admin/config/people/force_password_change/list/authenticated');

		$container->set('url_generator', $this->urlGenerator);

		\Drupal::setContainer($container);

		$this->form = AdminForm::create($container);
	}

	/**
	 * @covers:getFormId
	 */
	public function testGetFormId()
	{
		$this->assertEquals('force_password_change_admin_form', $this->form->getFormId(), 'Form ID is correct');
	}

	/**
	 * @covers ::buildForm
	 */
	public function testBuildForm()
	{
		$this->configFactory->expects($this->at(0))
			->method('get')
			->with('force_password_change.first_time_login_password_change')
			->willReturn($this->configFactory);
		$this->configFactory->expects($this->at(1))
			->method('get')
			->with('change')
			->willReturn(TRUE);
		$this->configFactory->expects($this->at(2))
			->method('get')
			->with('force_password_change.when_to_check')
			->willReturn($this->configFactory);
		$this->configFactory->expects($this->at(3))
			->method('get')
			->with('login_only')
			->willReturn(TRUE);
		$this->configFactory->expects($this->at(4))
			->method('get')
			->with('force_password_change.expire_password')
			->willReturn($this->configFactory);
		$this->configFactory->expects($this->at(5))
			->method('get')
			->with('expire')
			->willReturn(TRUE);

		$form = $this->formBuilder->getForm($this->form);

		$this->assertArrayHasKey('#attached', $form, 'The form has the #attached element');
		$this->assertArrayHasKey('library', $form['#attached'], 'The #attached element has the library key');
		$this->assertArrayHasKey(0, $form['#attached']['library'], 'A library has been attached to the form');
		$this->assertEquals('force_password_change/settings_page', $form['#attached']['library'][0], 'The force_password_change/settings_page library is attached to the form');
		$this->assertArrayHasKey('first_time_login_password_change', $form, 'The first_time_login_password_change key exists');
		$this->assertArrayHasKey('first_time_login_password_change', $form, 'The first_time_login_password_change key exists');
		$this->assertEquals('checkbox', $form['first_time_login_password_change']['#type'], 'first_time_login_password_change element is a checkbox');
		$this->assertArrayHasKey('login_only', $form, 'the login_only element exists in the form');
		$this->assertEquals('radios', $form['login_only']['#type'], 'The login_only element is of #type radios');
		$this->assertArrayHasKey('#options', $form['login_only'], 'The #options key exists for the login_only element');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $form['login_only']['#options'][0], 'Option 0 of login_only #options is a Drupal\Core\StringTranslation\TranslatableMarkup object');
		$this->assertEquals('On every page load', $form['login_only']['#options'][0]->getUntranslatedString(), 'Option 0 of login_only #options is the correct value');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $form['login_only']['#options'][1], 'Option 0 of login_only #options is a Drupal\Core\StringTranslation\TranslatableMarkup object');
		$this->assertEquals('On login only', $form['login_only']['#options'][1]->getUntranslatedString(), 'Option 1 of login_only #options is the correct value');
		$this->assertArrayHasKey('roles', $form, 'The roles element exists in the form');
		$this->assertEquals('checkboxes', $form['roles']['#type'], 'The roles element is of #type checkboxes');
		$this->assertArrayHasKey('#options', $form['roles'], 'The #options key exists for the roles element');
		$this->assertArrayHasKey('authenticated', $form['roles']['#options'], 'The authenticated role exists as an option in the roles element');
		$this->assertInstanceOf('Drupal\Core\StringTranslation\TranslatableMarkup', $form['roles']['#options']['authenticated'], 'Option authenticated of roles #options is a Drupal\Core\StringTranslation\TranslatableMarkup object');
		$this->assertEquals('@role (Users in role: @user_count | Users with pending forced password change: @pending_user_count | <a href=":url">Details</a>)', $form['roles']['#options']['authenticated']->getUntranslatedString(), 'The text for the authenticated role is correct in the #options of the roles element');
		$this->assertEquals('Authenticated (Users in role: 1 | Users with pending forced password change: 1 | <a href="/admin/config/people/force_password_change/list/authenticated">Details</a>)', $form['roles']['#options']['authenticated']->render(), 'The details string for the authenticated role details page is correct');
		$this->assertArrayHasKey('expiry_data', $form, 'The expiry_data element exists in the form');
		$this->assertEquals('value', $form['expiry_data']['#type'], 'The #type of the expiry_data element is value');
		$this->assertTrue(is_array($form['expiry_data']['#value']), 'The value of the expiry_data element contains an array');
		$this->assertArrayHasKey('authenticated', $form['expiry_data']['#value'], 'The authenticated role exists in the #value of the expiry_data element');
		$this->assertArrayHasKey('expiry', $form['expiry_data']['#value']['authenticated'], 'The expiry key exists for the authenticated role in the #value of the expiry_data element');
		$this->assertEquals(60 * 60 * 24,  $form['expiry_data']['#value']['authenticated']['expiry'], 'The value of the expiry for the authenticated role in the #$value of the expiry_data element is correct');
		$this->assertArrayHasKey('weight', $form['expiry_data']['#value']['authenticated'], 'The weight key exists for the authenticated role in the #value of the expiry_data element');
		$this->assertEquals(0,  $form['expiry_data']['#value']['authenticated']['weight'], 'The value of the weight for the authenticated role in the #$value of the expiry_data element is correct');
		$this->assertArrayHasKey('expiry', $form, 'The expiry element of the form exists');
		$this->assertEquals('fieldset', $form['expiry']['#type'], 'The type of the expiry element is a fieldset');
		$this->assertArrayHasKey('expire_password', $form['expiry'], 'The expire_password element exists in the form');
		$this->assertEquals('checkbox', $form['expiry']['expire_password']['#type'], 'The #type of the expire_password is of type checkbox');
		$this->assertArrayHasKey('table', $form['expiry'], 'The table element exists in the expiry fieldset');
		$this->assertTrue($form['expiry']['table']['#tree'], '#tree is set to true in the expiry table');
		$this->assertEquals('table', $form['expiry']['table']['#type'], 'The type of the expiry table is set to table');
		$this->assertEquals('force_password_change_role_expiry_table', $form['expiry']['table']['#id'], 'TheID of the expiry table is correct');
		$this->assertArrayHasKey('#tabledrag', $form['expiry']['table'], 'The expiry table has tabledrag enabled');
		$this->assertArrayHasKey('#weight', $form['expiry']['table']['authenticated'], 'The authenticated row of the expiry table has the #weight element');
		$this->assertArrayHasKey('#attributes', $form['expiry']['table']['authenticated'], 'The authenticated row of the expiry table has the #attributes element');
		$this->assertEquals('draggable', $form['expiry']['table']['authenticated']['#attributes']['class'][0], 'The draggable class has been properly applied to the row in the expiry table');
		$this->assertEquals('1', $form['expiry']['table']['authenticated']['time']['time_quantity']['#default_value'], 'The time quantity for the authenticated role is correct');
		$this->assertEquals('day', $form['expiry']['table']['authenticated']['time']['time_period']['#default_value'], 'The time period for the authenticated role is correct');
		$this->assertEquals('weight', $form['expiry']['table']['authenticated']['weight']['#type'], 'The #type the authenticated role weight is weight');
		$this->assertEquals('0', $form['expiry']['table']['authenticated']['weight']['#default_value'], 'The #deafult_value the authenticated role weight is correct');
		$this->assertEquals('submit', $form['actions']['submit']['#type'], 'The submit button for the form exists');
	}
}
