<?php

namespace Drupal\Tests\force_password_change\Form;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Form\FormState;
use Drupal\user\Entity\Role;
use Drupal\Tests\Core\Form\FormTestBase;
use Drupal\force_password_change\Form\RoleForceForm;

/**
 * @coversDefaultClass \Drupal\force_password_change\Form\RoleForceForm
 * @group force_password_change
 */
class RoleForceFormTest extends FormTestBase
{
	/**
	 * The RoleForceForm
	 */
	protected $form;

	/**
	 * The Config Factory
	 */
	protected $configFactory;

	/**
	 * The Date Formatter
	 */
	protected $dateFormatter;

	/**
	 * The Force Password Change Service
	 */
	protected $forcePasswordChangeService;

	/**
	 * {@inheritdoc}
	 */
	public function setUp()
	{
		parent::setUp();

		// Create a dummy container.
		$container = new ContainerBuilder();

		$container->set('string_translation', $this->getStringTranslationStub());

		$this->configFactory = $this->getMockBuilder('\Drupal\Core\Config\ConfigFactoryInterface')
			->disableOriginalConstructor()
			->getMock();

		$container->set('config.factory', $this->configFactory);

		$this->dateFormatter = $this->getMockBuilder('\Drupal\Core\Datetime\DateFormatter')
			->disableOriginalConstructor()
			->getMock();

		$container->set('date.formatter', $this->dateFormatter);

		$this->forcePasswordChangeService = $this->getMockBuilder('\Drupal\force_password_change\Service\ForcePasswordChangeServiceInterface')
			->disableOriginalConstructor()
			->getMock();

		$container->set('force_password_change.service', $this->forcePasswordChangeService);

		$role = $this->getMockBuilder('\Drupal\user\Entity\Role')
			->disableOriginalConstructor()
			->getMock();
		$role->expects($this->any())
			->method('id')
			->willReturn('authenticated');
		$role->expects($this->any())
			->method('label')
			->willReturn('Authenticated');

		$roleStorage = $this->getMockBuilder('\Drupal\user\RoleStorageInterface')
			->disableOriginalConstructor()
			->getMock();
		$roleStorage->expects($this->any())
			->method('load')
			->will($this->returnValueMap([
				['authenticated', $role],
			]));
		$roleStorage->expects($this->any())
			->method('create')
			->willReturn($role);

		$entityManager = $this->getMockBuilder('Drupal\Core\Entity\EntityManagerInterface')
			->disableOriginalConstructor()
			->getMock();
		$entityManager->expects($this->any())
			->method('getStorage')
			->with(NULL)
			->willReturn($roleStorage);

		$container->set('entity.manager', $entityManager);

		\Drupal::setContainer($container);

		$this->form = RoleForceForm::create($container);
	}

	/**
	 * @covers:getFormId
	 */
	public function testGetFormId()
	{
		$this->assertEquals('force_password_change_role_admin_form', $this->form->getFormId(), 'Form ID is correct');
	}

	/**
	 * @covers ::buildForm
	 */
	public function testBuildForm()
	{
		$form = $this->formBuilder->getForm($this->form);

		$this->assertArrayHasKey('no_role', $form, 'no_role elements exists in form when role is not set');

		$this->configFactory->expects($this->at(0))
			->method('get')
			->with('force_password_change.when_to_check')
			->willReturn($this->configFactory);
		$this->configFactory->expects($this->at(1))
			->method('get')
			->with('login_only')
			->willReturn(TRUE);

		$role = Role::create(['id' => 'authenticated', 'label' => 'Authenticated']);

		$form = $this->formBuilder->getForm($this->form, $role);

		$this->assertArrayNotHasKey('no_role', $form, 'no_role element does not exist in form when role is set');
		$this->assertArrayHasKey('force_password_change', $form, 'force_password_change element exists in form');
		$this->assertEquals('checkbox', $form['force_password_change']['#type'], 'The #type of the force_password_change element is checkbox');
		$this->assertArrayHasKey('role', $form, 'The role element exists in the form');
		$this->assertEquals('value', $form['role']['#type'], 'The #type of the role element is value');
		$this->assertEquals($role, $form['role']['#value'], 'The #value of the role element is the role');
		$this->assertArrayHasKey('submit', $form['actions'], 'The submit element exists in $form[actions]');
		$this->assertEquals('submit', $form['actions']['submit']['#type'], 'The submit element is of #type submit');
	}
}
